@@ -147,11 +147,11 @@ async def test_browser_state_capture(manager: BrowserManager):
     )
     os.makedirs("./tmp/screenshots", exist_ok=True)
     with open("./tmp/screenshots/agent1_python.png", "wb") as f:
-        f.write(base64.b64decode(screenshot1))
+        f.write(screenshot1)
     with open("./tmp/screenshots/agent2_rust.png", "wb") as f:
-        f.write(base64.b64decode(screenshot2))
+        f.write(screenshot2)
     with open("./tmp/screenshots/agent3_github.png", "wb") as f:
-        f.write(base64.b64decode(screenshot3))
+        f.write(screenshot3)
     end_time = time.time()
     total_duration = end_time - start_time
     logging.info(f"🔍 DIAGNOSIS: All taking screenshot completed in {total_duration:.3f}s")
@@ -215,7 +215,7 @@ async def test_browser_state_capture(manager: BrowserManager):
     # Get new state for agent 1
     logging.info("Capturing new state for agent 1...")
     state1_new = await agent1.get_browser_state_summary()
-
+    pdb.set_trace()
     # Print new tabs and save screenshot
     logging.info(f"Agent 1 new tabs: {state1_new.tabs}")
     assert state1_new.screenshot, "Agent 1 failed to capture new screenshot."
@@ -227,12 +227,12 @@ async def test_browser_state_capture(manager: BrowserManager):
     logging.info("Verifying active page with manager...")
     active_target_id = await manager._get_active_target()
     logging.info(f"Manager sees active target: {active_target_id}")
-    active_tab1 = await manager._get_activate_tab_info()
+    active_tab1 = await manager.get_activate_tab()
     print(active_tab1)
     await agent2.active_focus_page()
     active_target_id = await manager._get_active_target()
     logging.info(f"After switching, Manager sees active target: {active_target_id}")
-    active_tab2 = await manager._get_activate_tab_info()
+    active_tab2 = await manager.get_activate_tab()
     print(active_tab2)
     await manager.unregister_agent("agent-state-1", close_tabs=True)
     main_tabs = await manager.main_browser_session.get_tabs()
@@ -331,6 +331,39 @@ async def test_website_api(main_browser_session: AgentBrowserSession):
     # pdb.set_trace()
 
 
+async def test_page_element(browser_session: AgentBrowserSession):
+    # await browser_session.navigate_to_url("https://github.com/", new_tab=True)
+    # await asyncio.sleep(1)
+    # page = await browser_session.get_current_page()
+    # css_selector = r"#FormControl--\:Rjqhb\: > div > button"
+    # element = await page.get_elements_by_css_selector(css_selector)
+    # await page.get_element()
+    # mouse = await page.mouse
+    # await mouse.scroll(x=0, y=100, delta_x=0, delta_y=1000)
+    # await page.press("Enter")
+    #
+    # if element:
+    #     await element[0].click()
+    #     await element[0].click(button='left', click_count=1, modifiers=['Control'])
+    #     await element[0].fill("Hello World")
+
+    from vibe_surf.browser.find_page_element import SemanticExtractor
+
+    semantic_extractor = SemanticExtractor()
+    await browser_session.navigate_to_url("https://www.google.com/search?q=langflow/", new_tab=True)
+    await browser_session._wait_for_stable_network()
+
+    page = await browser_session.get_current_page()
+
+    element_mappings = await semantic_extractor.extract_semantic_mapping(page)
+    element_info = semantic_extractor.find_element_by_hierarchy(element_mappings, target_text="新闻")
+    element = await page.get_elements_by_css_selector(element_info["hierarchical_selector"] or element_info["selectors"])
+    if element:
+        await element[0].click()
+    pdb.set_trace()
+
+
+
 async def main():
     """
     Main function to run all browser session tests.
@@ -364,9 +397,10 @@ async def main():
             # await test_manual_page_assignment(manager)
             # await test_agent_cleanup(manager)
             # await test_agent_tab_isolation(manager)
-            # await test_browser_state_capture(manager)
+            await test_browser_state_capture(manager)
             # await get_all_css_selector(main_browser_session)
-            await test_website_api(main_browser_session)
+            # await test_website_api(main_browser_session)
+            # await test_page_element(browser_session=main_browser_session)
 
     except Exception as e:
         logging.error(f"An error occurred during tests: {e}", exc_info=True)
