@@ -39,4 +39,4 @@ def configure_system_proxies():
         logger.info(f"Set HTTPS_PROXY to: {https_proxy}")
 
     if http_proxy or https_proxy:
-        os.environ['no_proxy'] = 'localhost,127.0.0.1,::1,backend.composio.dev'
\ No newline at end of file
+        os.environ['no_proxy'] = 'localhost,127.0.0.1,::1,backend.composio.dev,us.i.posthog.com'
\ No newline at end of file
