@@ -30,12 +30,12 @@ except ImportError:
 
 # Logo components with styling for rich panels
 VIBESURF_LOGO = """
-[white]██╗   ██╗██╗██████╗ ███████╗[/]    [darkorange]███████╗██╗   ██╗██████╗ ███████╗[/]
-[white]██║   ██║██║██╔══██╗██╔════╝[/]    [darkorange]██╔════╝██║   ██║██╔══██╗██╔════╝[/]
-[white]██║   ██║██║██████╔╝█████╗  [/]    [darkorange]███████╗██║   ██║██████╔╝█████╗  [/]
-[white]╚██╗ ██╔╝██║██╔══██╗██╔══╝  [/]    [darkorange]╚════██║██║   ██║██╔══██╗██╔══╝  [/]
-[white] ╚████╔╝ ██║██████╔╝███████╗[/]    [darkorange]███████║╚██████╔╝██║  ██║██║     [/]
-[white]  ╚═══╝  ╚═╝╚═════╝ ╚══════╝[/]    [darkorange]╚══════╝ ╚═════╝ ╚═╝  ╚═╝╚═╝     [/]
+[bright_cyan]██╗   ██╗██╗██████╗ ███████╗[/]    [cyan]███████╗██╗   ██╗██████╗ ███████╗[/]
+[bright_cyan]██║   ██║██║██╔══██╗██╔════╝[/]    [cyan]██╔════╝██║   ██║██╔══██╗██╔════╝[/]
+[bright_cyan]██║   ██║██║██████╔╝█████╗  [/]    [cyan]███████╗██║   ██║██████╔╝█████╗  [/]
+[bright_cyan]╚██╗ ██╔╝██║██╔══██╗██╔══╝  [/]    [cyan]╚════██║██║   ██║██╔══██╗██╔══╝  [/]
+[bright_cyan] ╚████╔╝ ██║██████╔╝███████╗[/]    [cyan]███████║╚██████╔╝██║  ██║██║     [/]
+[bright_cyan]  ╚═══╝  ╚═╝╚═════╝ ╚══════╝[/]    [cyan]╚══════╝ ╚═════╝ ╚═╝  ╚═╝╚═╝     [/]
 """
 
 console = Console()
@@ -454,7 +454,7 @@ def main():
         
         # Display logo
         console.print(Panel(VIBESURF_LOGO, title="[bold cyan]VibeSurf CLI[/bold cyan]", border_style="cyan"))
-        console.print("[dim]A powerful browser automation tool for vibe surfing 🏄‍♂️[/dim]")
+        console.print("[dim]A powerful browser automation tool for vibe surfing.")
         import vibe_surf
         console.print(f"[dim]Version: {vibe_surf.__version__}[/dim]\n")
         console.print(f"[dim]Author: WarmShao and Community Contributors [/dim]\n")
