@@ -6,6 +6,10 @@ from passlib.context import CryptContext
 from pydantic import Field, SecretStr, field_validator
 from pydantic_settings import BaseSettings, SettingsConfigDict
 
+# Diagnostic logging for bcrypt import issues
+import logging
+logger_diag = logging.getLogger(__name__)
+
 from vibe_surf.langflow.logging.logger import logger
 from vibe_surf.langflow.services.settings.constants import DEFAULT_SUPERUSER, DEFAULT_SUPERUSER_PASSWORD
 from vibe_surf.langflow.services.settings.utils import read_secret_from_file, write_secret_to_file
