@@ -10,6 +10,7 @@ from vibe_surf.logger import get_logger
 
 logger = get_logger(__name__)
 
+
 class PythonFile(BaseFile):
     """Plain text file implementation"""
 
@@ -73,6 +74,18 @@ class CustomFileSystem(FileSystem):
 
         return file_content
 
+    def get_todo_contents(self) -> str:
+        """Get todo file contents"""
+        full_filepath = str(self.data_dir / "todo.md")
+        if not os.path.exists(full_filepath):
+            return f"TODO '{full_filepath}' not found."
+        try:
+            with open(str(full_filepath), 'r', encoding="utf-8") as f:
+                todo_content = f.read()
+            return todo_content
+        except Exception as e:
+            return ""
+
     async def read_file(self, full_filename: str, external_file: bool = False) -> str:
         """Read file content using file-specific read method and return appropriate message to LLM"""
         try:
@@ -88,7 +101,7 @@ class CustomFileSystem(FileSystem):
                 with open(str(full_filepath), 'r', encoding="utf-8") as f:
                     content = f.read()
                     return f'Read from file {full_filename}.\n<content>\n{content}\n</content>'
-                
+
             elif extension == 'pdf':
                 import pypdf
 
@@ -254,23 +267,9 @@ class CustomFileSystem(FileSystem):
 
     async def write_file(self, full_filename: str, content: str) -> str:
         """Write content to file using file-specific write method"""
-        if not self._is_valid_filename(full_filename):
-            return INVALID_FILENAME_ERROR_MESSAGE
-
         try:
             full_path = self.data_dir / full_filename
             full_path.parent.mkdir(parents=True, exist_ok=True)
-            name_without_ext, extension = self._parse_filename(full_filename)
-            file_class = self._get_file_type_class(extension)
-            if not file_class:
-                raise ValueError(f"Error: Invalid file extension '{extension}' for file '{full_filename}'.")
-
-            # Create or get existing file using full filename as key
-            if full_filename in self.files:
-                file_obj = self.files[full_filename]
-            else:
-                file_obj = file_class(name=name_without_ext)
-                self.files[full_filename] = file_obj  # Use full filename as key
 
             with open(str(full_path), encoding='utf-8', mode='w') as f:
                 f.write(content)
@@ -293,17 +292,6 @@ class CustomFileSystem(FileSystem):
         try:
             full_path = self.data_dir / full_filename
             full_path.parent.mkdir(parents=True, exist_ok=True)
-            name_without_ext, extension = self._parse_filename(full_filename)
-            file_class = self._get_file_type_class(extension)
-            if not file_class:
-                raise ValueError(f"Error: Invalid file extension '{extension}' for file '{full_filename}'.")
-
-            # Create or get existing file using full filename as key
-            if full_filename in self.files:
-                file_obj = self.files[full_filename]
-            else:
-                file_obj = file_class(name=name_without_ext)
-                self.files[full_filename] = file_obj  # Use full filename as key
 
             # Use file-specific write method
             with open(str(full_path), encoding='utf-8', mode='w') as f:
