@@ -10,6 +10,22 @@ import base64
 import mimetypes
 import yfinance as yf
 import pprint
+import pandas as pd
+import numpy as np
+import matplotlib
+
+matplotlib.use('Agg')  # Use non-interactive backend
+import matplotlib.pyplot as plt
+import seaborn as sns
+import io
+import json
+import re
+import os
+import openpyxl
+from datetime import datetime, timedelta
+from pathlib import Path
+import csv
+
 from json_repair import repair_json
 from datetime import datetime
 from typing import Optional, Type, Callable, Dict, Any, Union, Awaitable, TypeVar
@@ -33,7 +49,7 @@ from vibe_surf.tools.views import HoverAction, ExtractionAction, FileExtractionA
     ReportWriterTask, TodoGenerateAction, TodoModifyAction, VibeSurfDoneAction, SkillSearchAction, SkillCrawlAction, \
     SkillSummaryAction, SkillTakeScreenshotAction, SkillDeepResearchAction, SkillCodeAction, SkillFinanceAction, \
     SkillXhsAction, SkillDouyinAction, SkillYoutubeAction, SkillWeiboAction, GrepContentAction, \
-    SearchToolAction, GetToolInfoAction, ExecuteExtraToolAction
+    SearchToolAction, GetToolInfoAction, ExecuteExtraToolAction, ExecutePythonCodeAction
 from vibe_surf.tools.finance_tools import FinanceDataRetriever, FinanceMarkdownFormatter, FinanceMethod
 from vibe_surf.tools.mcp_client import CustomMCPClient
 from vibe_surf.tools.composio_client import ComposioClient
@@ -43,7 +59,8 @@ from vibe_surf.tools.vibesurf_registry import VibeSurfRegistry
 from bs4 import BeautifulSoup
 from vibe_surf.logger import get_logger
 from vibe_surf.tools.utils import clean_html_basic
-from vibe_surf.tools.utils import _extract_structured_content, _rank_search_results_with_llm, extract_file_content_with_llm
+from vibe_surf.tools.utils import _extract_structured_content, _rank_search_results_with_llm, \
+    extract_file_content_with_llm, remove_import_statements
 
 logger = get_logger(__name__)
 
@@ -385,11 +402,11 @@ class VibeSurfTools:
                 params: SkillCodeAction,
                 browser_manager: BrowserManager,
                 page_extraction_llm: BaseChatModel,
+                file_system: CustomFileSystem
         ):
             """
             Skill: Generate and execute JavaScript code from functional requirements or code prompts with iterative retry logic
             """
-            MAX_ITERATIONS = 5
 
             try:
                 if not page_extraction_llm:
@@ -408,7 +425,16 @@ class VibeSurfTools:
                 success, execute_result, js_code = await generate_java_script_code(params.code_requirement,
                                                                                    page_extraction_llm, browser_session,
                                                                                    MAX_ITERATIONS=5)
-                msg = f'```javascript\n{js_code}\n```\nResult:\n```json\n {execute_result}\n```\n'
+                timestamp = datetime.now().strftime("%Y%m%d_%H%M%S")
+                code_file = f"codes/{timestamp}.js"
+                await file_system.write_file(code_file, js_code)
+
+                if len(execute_result) < 1000:
+                    msg = f'JavaScript Code save at:{code_file}\nResult:\n```json\n {execute_result}\n```\n'
+                else:
+                    result_file = f"codes/{timestamp}.json"
+                    await file_system.write_file(result_file, execute_result)
+                    msg = f'Code save at:{code_file}\n```javascript\n{js_code}\n```\n Result save at {result_file}:\n```json\n {execute_result[:1000]}\n...\n```'
                 if success:
                     return ActionResult(extracted_content=msg)
                 else:
@@ -923,11 +949,165 @@ class VibeSurfTools:
                 logger.error(error_msg)
                 return ActionResult(error=error_msg, extracted_content=error_msg)
 
+        @self.registry.action(
+            """Execute Python code for data processing, visualization, and analysis.
+            """,
+            param_model=ExecutePythonCodeAction,
+        )
+        async def execute_python_code(
+                params: ExecutePythonCodeAction,
+                file_system: CustomFileSystem
+        ):
+            try:
+                # Get base directory from file system
+                base_dir = str(file_system.get_dir())
+                
+                # Create a secure namespace with pre-imported libraries
+                namespace = {
+                    '__builtins__': {
+                        # Safe built-ins only
+                        'abs': abs, 'all': all, 'any': any, 'bin': bin, 'bool': bool,
+                        'chr': chr, 'dict': dict, 'dir': dir, 'divmod': divmod,
+                        'enumerate': enumerate, 'filter': filter, 'float': float,
+                        'format': format, 'frozenset': frozenset, 'getattr': getattr,
+                        'hasattr': hasattr, 'hash': hash, 'hex': hex, 'id': id,
+                        'int': int, 'isinstance': isinstance, 'issubclass': issubclass,
+                        'iter': iter, 'len': len, 'list': list, 'map': map,
+                        'max': max, 'min': min, 'next': next, 'oct': oct,
+                        'ord': ord, 'pow': pow, 'print': print, 'range': range,
+                        'repr': repr, 'reversed': reversed, 'round': round,
+                        'set': set, 'setattr': setattr, 'slice': slice,
+                        'sorted': sorted, 'str': str, 'sum': sum, 'tuple': tuple,
+                        'type': type, 'zip': zip,
+                    },
+                    'SAVE_DIR': base_dir,
+                }
+                
+                # Import common libraries safely
+                try:
+
+                    # Add libraries to namespace
+                    namespace.update({
+                        'pd': pd,
+                        'pandas': pd,
+                        'np': np,
+                        'numpy': np,
+                        'plt': plt,
+                        'matplotlib': matplotlib,
+                        'sns': sns,
+                        'seaborn': sns,
+                        'json': json,
+                        're': re,
+                        'os': os,
+                        'openpyxl': openpyxl,
+                        'datetime': datetime,
+                        'timedelta': timedelta,
+                        'Path': Path,
+                        'csv': csv,
+                        'io': io
+                    })
+                    
+                    # Add secure file helper functions
+                    def safe_open(path, mode='r', **kwargs):
+                        """Secure file open that restricts operations to BASE_DIR"""
+                        if (not path.startswith(base_dir)) and os.path.isabs(path):
+                            raise PermissionError("Absolute paths are not allowed. Only relative paths within workspace are supported.")
+                        if not path.startswith(base_dir):
+                            full_path = os.path.join(base_dir, path)
+                        else:
+                            full_path = path
+                        if not full_path.startswith(base_dir):
+                            raise PermissionError("File operations are restricted to workspace directory only.")
+                        os.makedirs(os.path.dirname(full_path), exist_ok=True)
+                        return open(full_path, mode, **kwargs)
+                    
+                    def safe_path(path):
+                        """Get safe path within BASE_DIR"""
+                        if (not path.startswith(base_dir)) and os.path.isabs(path):
+                            raise PermissionError(
+                                "Absolute paths are not allowed. Only relative paths within workspace are supported.")
+                        if not path.startswith(base_dir):
+                            full_path = os.path.join(base_dir, path)
+                        else:
+                            full_path = path
+                        if not full_path.startswith(base_dir):
+                            raise PermissionError("File operations are restricted to workspace directory only.")
+                        return full_path
+                    
+                    namespace.update({
+                        'open': safe_open,
+                        'safe_path': safe_path,
+                    })
+                    
+                except ImportError as e:
+                    logger.warning(f"Failed to import some libraries: {e}")
+                
+                # Remove import statements from user code since modules are pre-imported
+                cleaned_code = remove_import_statements(params.code)
+                logger.info(cleaned_code)
+                
+                # Check for dangerous operations
+                dangerous_keywords = [
+                    'import subprocess', 'import sys', 'import importlib',
+                    '__import__', 'eval(', 'exec(', 'compile(',
+                    'open(', 'file(', 'input(', 'raw_input(',
+                    'execfile', 'reload', 'vars(', 'globals(', 'locals(',
+                    'delattr', 'setattr', 'getattr', '__'
+                ]
+                
+                code_lower = cleaned_code.lower()
+                for keyword in dangerous_keywords:
+                    if keyword in code_lower and keyword not in ['open(', '__']:  # Allow our safe open
+                        if keyword == 'open(' and 'safe_open' in cleaned_code:
+                            continue  # Allow our safe open function
+                        return ActionResult(
+                            error=f"🚫 Security Error: Dangerous operation '{keyword}' detected. Code execution blocked for security reasons."
+                        )
+                
+                # Capture stdout to get print outputs
+                import sys
+                from io import StringIO
+                
+                old_stdout = sys.stdout
+                sys.stdout = captured_output = StringIO()
+                
+                try:
+                    # Compile and execute the cleaned code
+                    compiled_code = compile(cleaned_code, '<code>', 'exec')
+                    exec(compiled_code, namespace, namespace)
+                    
+                    # Get the captured output
+                    output_value = captured_output.getvalue()
+                    if output_value.strip():
+                        result_text = output_value
+                    else:
+                        result_text = "No output printed to console after executing the Python code."
+                finally:
+                    # Restore stdout
+                    sys.stdout = old_stdout
+                
+                logger.info(f'🐍 Python code executed successfully')
+                return ActionResult(
+                    extracted_content=result_text)
+                
+            except PermissionError as e:
+                error_msg = f'🚫 Security Error: {str(e)}'
+                logger.error(error_msg)
+                return ActionResult(error=error_msg, extracted_content=error_msg)
+            except SyntaxError as e:
+                error_msg = f'❌ Syntax Error in Python code: {str(e)}'
+                logger.error(error_msg)
+                return ActionResult(error=error_msg, extracted_content=error_msg)
+            except Exception as e:
+                error_msg = f'❌ Python code execution failed: {str(e)}'
+                logger.error(error_msg)
+                return ActionResult(error=error_msg, extracted_content=error_msg)
+
     def _register_extra_tools(self):
         """
         Register extra tools for dynamic toolkit and MCP tool access
         """
-        
+
         @self.registry.action(
             'Get all available toolkit types from both Composio and MCP clients',
             param_model=NoParamsAction,
@@ -941,41 +1121,41 @@ class VibeSurfTools:
             """
             try:
                 toolkit_types = []
-                
+
                 # Add Composio toolkit types
                 if self.composio_toolkits:
                     toolkit_types.extend(list(self.composio_toolkits.keys()))
-                
+
                 # Add MCP client types
                 if self.mcp_clients:
                     toolkit_types.extend(list(self.mcp_clients.keys()))
-                
+
                 result_text = f"Available toolkit types ({len(toolkit_types)} total):\n\n"
-                
+
                 # Group by type
                 composio_types = list(self.composio_toolkits.keys()) if self.composio_toolkits else []
                 mcp_types = list(self.mcp_clients.keys()) if self.mcp_clients else []
-                
+
                 if composio_types:
                     result_text += f"**Composio Toolkits ({len(composio_types)}):**\n"
                     for toolkit in composio_types:
                         result_text += f"- {toolkit}\n"
                     result_text += "\n"
-                
+
                 if mcp_types:
                     result_text += f"**MCP Clients ({len(mcp_types)}):**\n"
                     for client in mcp_types:
                         result_text += f"- {client}\n"
                     result_text += "\n"
-                
+
                 if not toolkit_types:
                     result_text = "No toolkit types available. Please register Composio toolkits or MCP clients first."
-                
+
                 logger.info(f'📋 Retrieved {len(toolkit_types)} toolkit types')
                 return ActionResult(
                     extracted_content=result_text,
                 )
-                
+
             except Exception as e:
                 logger.error(f'❌ Failed to get toolkit types: {e}')
                 return ActionResult(error=f'Failed to get toolkit types: {str(e)}')
@@ -1000,9 +1180,9 @@ class VibeSurfTools:
                     filters = [f.lower() for f in params.filters]
                 else:
                     filters = []
-                
+
                 matching_tools = []
-                
+
                 # Search in registry actions with prefixes
                 for action_name, action in self.registry.registry.actions.items():
                     # Check if this action belongs to the specified toolkit type
@@ -1013,7 +1193,7 @@ class VibeSurfTools:
                             description = param_dict.get('description', action_name)
                         except:
                             description = action_name
-                        
+
                         # Check if any filter matches tool name or description
                         search_text = f"{action_name} {description}".lower()
                         if any(filter_term in search_text for filter_term in filters):
@@ -1021,7 +1201,7 @@ class VibeSurfTools:
                                 'tool_name': action_name,
                                 'description': description
                             })
-                    
+
                     elif toolkit_type in self.mcp_clients and action_name.startswith(f"mcp.{toolkit_type}."):
                         # Get tool description from param_model
                         try:
@@ -1029,7 +1209,7 @@ class VibeSurfTools:
                             description = param_dict.get('description', action_name)
                         except:
                             description = action_name
-                        
+
                         # Check if any filter matches tool name or description
                         search_text = f"{action_name} {description}".lower()
                         if any(filter_term in search_text for filter_term in filters):
@@ -1037,7 +1217,7 @@ class VibeSurfTools:
                                 'tool_name': action_name,
                                 'description': description
                             })
-                
+
                 # Format results
                 if matching_tools:
                     result_text = f"Found {len(matching_tools)} tools in '{toolkit_type}' matching filters {params.filters}:\n\n"
@@ -1046,14 +1226,14 @@ class VibeSurfTools:
                         result_text += f"   Description: {tool['description']}\n\n"
                 else:
                     result_text = f"No tools found in '{toolkit_type}' matching filters {params.filters}"
-                
+
                 logger.info(f'🔍 Found {len(matching_tools)} tools in {toolkit_type} matching filters')
                 return ActionResult(
                     extracted_content=result_text,
                     include_extracted_content_only_once=True,
                     long_term_memory=f'Found {len(matching_tools)} tools in {toolkit_type} matching filters: {", ".join(params.filters)}'
                 )
-                
+
             except Exception as e:
                 logger.error(f'❌ Failed to search tools: {e}')
                 return ActionResult(error=f'Failed to search tools: {str(e)}')
@@ -1074,24 +1254,24 @@ class VibeSurfTools:
             """
             try:
                 tool_name = params.tool_name
-                
+
                 if tool_name not in self.registry.registry.actions:
                     return ActionResult(error=f'Tool "{tool_name}" not found in registry')
-                
+
                 action = self.registry.registry.actions[tool_name]
-                
+
                 # Convert param_model to dict
                 try:
                     param_dict = action.param_model.model_json_schema()
                     result_text = json.dumps(param_dict, indent=2, ensure_ascii=False)
                 except Exception as e:
                     result_text = f"Tool: {tool_name}\nError getting parameter info: {str(e)}"
-                
+
                 logger.info(f'ℹ️ Retrieved tool info for: {tool_name}')
                 return ActionResult(
                     extracted_content=f"```json\n{result_text}\n```"
                 )
-                
+
             except Exception as e:
                 logger.error(f'❌ Failed to get tool info: {e}')
                 return ActionResult(error=f'Failed to get tool info: {str(e)}')
@@ -1120,10 +1300,10 @@ class VibeSurfTools:
             """
             try:
                 tool_name = params.tool_name
-                
+
                 if tool_name not in self.registry.registry.actions:
                     return ActionResult(error=f'Tool "{tool_name}" not found in registry')
-                
+
                 # Parse tool parameters
                 try:
                     tool_params_dict = json.loads(params.tool_params)
@@ -1132,36 +1312,37 @@ class VibeSurfTools:
                         tool_params_dict = json.loads(repair_json(params.tool_params))
                     except Exception as e:
                         return ActionResult(error=f'Failed to parse tool parameters: {str(e)}')
-                
+
                 # Get the action
                 action = self.registry.registry.actions[tool_name]
-                
+
                 # Create special context (same as in act method)
                 special_context = {
                     'browser_manager': browser_manager,
                     'page_extraction_llm': page_extraction_llm,
                     'file_system': file_system,
                 }
-                
+
                 # Validate parameters
                 try:
                     validated_params = action.param_model(**tool_params_dict)
                 except Exception as e:
-                    return ActionResult(error=f'Invalid parameters {tool_params_dict} for action {tool_name}: {type(e)}: {e}')
-                
+                    return ActionResult(
+                        error=f'Invalid parameters {tool_params_dict} for action {tool_name}: {type(e)}: {e}')
+
                 # Execute the tool
                 result = await action.function(params=validated_params, **special_context)
-                
+
                 logger.info(f'🔧 Successfully executed extra tool: {tool_name}')
                 return result
-                
+
             except Exception as e:
                 logger.error(f'❌ Failed to execute extra tool: {e}')
                 return ActionResult(error=f'Failed to execute extra tool: {str(e)}')
 
     def _register_browser_use_agent(self):
         @self.registry.action(
-            'Execute browser_use agent tasks. Please specify a tab id to an agent, if you want to let agent work on this tab.',
+            'Execute browser_use agent tasks. Please specify a tab id to an agent, if you want to let agent work on this tab. When using Parallel Task Processing, each `tab_id` in parameter must be unique - one tab_id can only be assigned to one agent during parallel execution. Otherwise, please use single bu agent to complete the task.',
             param_model=BrowserUseAgentExecution,
         )
         async def execute_browser_use_agent(
@@ -1424,7 +1605,7 @@ class VibeSurfTools:
             )
 
         @self.registry.action(
-            'Extract content from a file. Support image files, pdf, markdown, txt, json, csv.',
+            'Extract content from a file. Support images, pdf, markdown, txt, json, csv, python, js and etc..',
             param_model=FileExtractionAction,
         )
         async def extract_content_from_file(
