@@ -40,7 +40,7 @@ class BrowserUseAgentTask(BaseModel):
     """Parameters for a single browser_use agent task"""
     tab_id: str | None = Field(
         default=None,
-        description='Tab ID to execute the task on. If None, a new blank page will be created',
+        description='Tab ID to execute the task on. If None, a new blank page will be created. Must be unique in parallel tasks.',
     )
     task: str = Field(
         description='Task description focusing on what needs to be done, goals, and expected returns.',
@@ -55,10 +55,11 @@ class BrowserUseAgentExecution(BaseModel):
     """Parameters for executing browser_use agent tasks in parallel"""
     tasks: list[BrowserUseAgentTask] = Field(
         description='List of tasks to execute concurrently using browser_use agents for improved efficiency. '
-                    'If only one task and no tab_id is provided, the agent can take over the entire browser and can also see and operate all tabs.',
+                    'If only one task and no tab_id is provided, the agent can take over the entire browser and can also see and operate all tabs. ',
         min_length=1,
     )
 
+
 class BrowserUseFile(BaseModel):
     file_path: str = Field(description='Path to the file')
     file_description: str = Field(
@@ -98,6 +99,7 @@ class TodoModification(BaseModel):
         description='Text of the todo item to operate on',
     )
 
+
 class TodoModifyAction(BaseModel):
     """Parameters for modifying todo items"""
     modifications: list[TodoModification] = Field(
@@ -106,7 +108,6 @@ class TodoModifyAction(BaseModel):
     )
 
 
-
 class VibeSurfDoneAction(BaseModel):
     """Parameters for task completion output"""
     response: str = Field(
@@ -179,6 +180,13 @@ class SkillCodeAction(BaseModel):
     )
 
 
+class GenJSCodeAction(BaseModel):
+    """Parameters for skill_code action"""
+    code_requirement: str = Field(
+        description='Functional requirement or code prompt describing what the JavaScript code should accomplish. Can be a description like "extract products with price over $100", requirements, or complete/incomplete JavaScript code snippets that will be processed by LLM to generate proper executable code.',
+    )
+
+
 class SkillFinanceAction(BaseModel):
     """Parameters for skill_finance action"""
     symbol: str = Field(
@@ -221,7 +229,7 @@ class DownloadMediaAction(BaseModel):
         default=None,
         description='Optional custom filename. If not provided, will auto-detect from URL or Content-Disposition header',
     )
- 
+
 
 class SkillXhsAction(BaseModel):
     """Parameters for skill_xhs action - Xiaohongshu API skill"""
@@ -329,3 +337,10 @@ class ExecuteExtraToolAction(BaseModel):
     tool_params: str = Field(
         description='JSON string containing parameters for the tool execution',
     )
+
+
+class ExecutePythonCodeAction(BaseModel):
+    """Parameters for execute_python_code action"""
+    code: str = Field(
+        description='Python code to execute. Supports data processing, visualization (matplotlib/seaborn), file operations, and analysis. All file operations are restricted to the workspace directory for security.',
+    )
